#! /usr/bin/env python

# import Nexus functions
from nexus import settings,job,run_project,get_machine,obj
from nexus import generate_physical_system
from nexus import generate_pwscf
from nexus import generate_pw2qmcpack
from nexus import generate_qmcpack,vmc,loop,linear,dmc

# Nexus settings
settings(
    pseudo_dir    = 'PSP/',
    runs          = '',
    results       = '',
    status_only   = 0,
    generate_only = 0,
    sleep         = 1,
    machine       = 'daint',
    account       = 's700'
    )

appdir = '/scratch/snx3000/amsmax/QMCPack/bin/'
dftjob  = job(sub_options='-C gpu',nodes=32,threads= 1,queue='normal',hours=4,app=appdir+'pw.x')
p2qjob  = job(sub_options='-C gpu',cores= 1,threads= 1,queue='normal',hours=4,app=appdir+'pw2qmcpack.x')
qmcjob  = job(sub_options='-C gpu',nodes=32,threads=12,queue='normal',hours=14,app=appdir+'qmcpack.cpu')
daint = get_machine('daint') # allow one job at a time (lab only)
daint.queue_size = 100

# run directory and pseudopotentials
directory = 'sc_phosphorus' # directory to perform runs
dft_pps   = ['P.BFD_p.upf']   # pwscf pseudopotentials
qmc_pps   = ['P.BFD_p.xml']   # qmcpack pseudopotentials


# specify k-point grids
kgrids = [(2,2,2),(3,3,3),(4,4,4)]

sims = []
first = True
for kgrid in kgrids:
    ks = '{0}{1}{2}'.format(*kgrid)

    sc_P = generate_physical_system( 
          structure = 'POSCAR' ,
          P        = 5 ,
          kgrid     = kgrid,
          kshift     = (.5,.5,.5),
          net_charge= 0,
          net_spin  = 0,
          )

    # generate converged scf
    if first:
        scf = generate_pwscf(
            identifier     = 'scf',
            path           = directory+'/scf',
            job            = dftjob,
            input_type     = 'scf',
            system         = sc_P,
            spin_polarized = False,
            pseudos        = dft_pps,
            input_dft      = 'lda', 
            ecut           = 300,
            conv_thr       = 1e-8, 
            mixing_beta    = .7,
            nosym          = False,
            wf_collect     = False,
            kgrid          = (12,12,12),
            kshift         = (0,0,0)
            )
        sims.append(scf)
    #end if

    # nscf run to generate orbitals
    nscf = generate_pwscf(
        identifier     = 'nscf',
        path           = directory+'/nscf-8at_'+ks,
        job            = dftjob,
        input_type     = 'nscf',
        system         = sc_P,
        spin_polarized = False,
        pseudos        = dft_pps,
        input_dft      = 'lda', 
        ecut           = 300,
        conv_thr       = 1e-8, 
        mixing_beta    = .7,
        tprnfor        = False,
        tstress        = False,
        nosym          = True,
        wf_collect     = True,
        dependencies   = (scf,'charge_density')
        )
    sims.append(nscf)

    p2q = generate_pw2qmcpack(
        identifier   = 'p2q',
        path         = directory+'/nscf-8at_'+ks,
        job          = p2qjob,
        write_psir   = False,
        dependencies = (nscf,'orbitals'),
        )
    sims.append(p2q)

    if first:
        # optimization inputs
        linopt1 = linear(
            energy               = 0.0,
            unreweightedvariance = 1.0,
            reweightedvariance   = 0.0,
            timestep             = 0.4,
            samples              = 8192,
            warmupsteps          = 50,
            blocks               = 64,
            substeps             = 4,
            nonlocalpp           = True,
            usebuffer            = True,
            walkers              = 1,
            minwalkers           = 0.5,
            maxweight            = 1e9, 
            usedrift             = True,
            minmethod            = 'quartic',
            beta                 = 0.0,
            exp0                 = -16,
            bigchange            = 15.0,
            alloweddifference    = 1e-4,
            stepsize             = 0.2,
            stabilizerscale      = 1.0,
            nstabilizers         = 3
            )

        linopt2 = linopt1.copy()
        linopt2.samples = 16384

        # optimization run
        opt = generate_qmcpack(
            identifier   = 'opt',
            path         = directory+'/opt-8at',
            job          = qmcjob,
            input_type   = 'basic',
            system       = sc_P,
            twistnum     = 0,
            bconds       = 'ppp',
            pseudos      = qmc_pps,
            jastrows     = [('J1','bspline',8),
                            ('J2','bspline',8)],
            calculations = [loop(max=4,qmc=linopt1),
                            loop(max=4,qmc=linopt2)],
            dependencies = (p2q,'orbitals')
            )
        sims.append(opt)
    #end if

    # DMC run
    ntwists = kgrid[0]*kgrid[1]*kgrid[2]
    qmc = generate_qmcpack(
        identifier   = 'dmc',
        path         = directory+'/dmc-8at_'+ks,
        job          = job(machine='daint',gpu=True,nodes=ntwists,hours=4,threads=12,processes_per_node=1,queue="normal",app=appdir+'qmcpack.cpu'),
        input_type   = 'basic',
        system       = sc_P,
        bconds       = 'ppp',
        pseudos      = qmc_pps,
        jastrows     = [],            
        calculations = [
            vmc(
                walkers     =   1,
                warmupsteps =  30,
                blocks      =  20,
                steps       =  10,
                substeps    =   2,
                timestep    =  .4,
                samples     = 2048
                ),
            dmc(
                warmupsteps   =  20, 
                blocks        =  100,
                steps         =  5,
                timestep      = 0.04,
                nonlocalmoves = True
                )
            ],
        dependencies = [(p2q,'orbitals'),(opt,'jastrow')]
        )
    sims.append(qmc)
    # DMC run
    qmcgpu = generate_qmcpack(
        identifier   = 'dmcgpu',
        path         = directory+'/dmcgpu-8at_'+ks,
        job          = job(machine='daint',gpu=True,nodes=ntwists,hours=4,threads=12,processes_per_node=1,queue="normal",app=appdir+'qmcpack.gpu.double'),
        input_type   = 'basic',
        system       = sc_P,
        bconds       = 'ppp',
        pseudos      = qmc_pps,
        jastrows     = [],            
        calculations = [
            vmc(
                walkers     =   128,
                warmupsteps =  30,
                blocks      =  20,
                steps       =  10,
                substeps    =   2,
                timestep    =  .4,
                samples     = 2048
                ),
            dmc(
                warmupsteps   =  20, 
                blocks        =  100,
                steps         =  5,
                timestep      = 0.04,
                nonlocalmoves = True
                )
            ],
        dependencies = [(p2q,'orbitals'),(opt,'jastrow')]
        )
    sims.append(qmcgpu)
    # VMC run
    vqmc = generate_qmcpack(
        identifier   = 'vmc',
        path         = directory+'/vmc-8at_'+ks,
        job          = job(machine='daint',gpu=True,nodes=ntwists,hours=1,threads=12,processes_per_node=1,queue="normal",app=appdir+'qmcpack.cpu'),
        input_type   = 'basic',
        system       = sc_P,
        bconds       = 'ppp',
        pseudos      = qmc_pps,
        jastrows     = [],            
        calculations = [
            vmc(
                walkers     =   128,
                warmupsteps =  30,
                blocks      =  1000,
                steps       =  10,
                substeps    =   2,
                timestep    =  .4,
                samples     = 2048
                )
            ],
        dependencies = [(p2q,'orbitals'),(opt,'jastrow')]
        )
    sims.append(vqmc)
    # VMC run
    vqmcgpu = generate_qmcpack(
        identifier   = 'vdmcgpu',
        path         = directory+'/vmcgpu-8at_'+ks,
        job          = job(machine='daint',gpu=True,nodes=ntwists,hours=4,threads=12,processes_per_node=1,queue="normal",app=appdir+'qmcpack.gpu.double'),
        input_type   = 'basic',
        system       = sc_P,
        bconds       = 'ppp',
        pseudos      = qmc_pps,
        jastrows     = [],            
        calculations = [
            vmc(
                walkers     =   128,
                warmupsteps =  30,
                blocks      =  1000,
                steps       =  10,
                substeps    =   2,
                timestep    =  .4,
                samples     = 2048
                )
            ],
        dependencies = [(p2q,'orbitals'),(opt,'jastrow')]
        )
    sims.append(vqmcgpu)
    
    first = False
#end for

# write input files and submit jobs
run_project(sims)

